//
//  ERAuthDelegateImpl.swift
//  sample
//
//  Created by Masayuki ISHII on 2019/01/29.
//  Copyright © 2019 veldt. All rights reserved.
//

import Foundation
import GoogleSignIn
//import FacebookCore
//import FacebookLogin
import FBSDKCoreKit
import FBSDKLoginKit
import SwiftyJSON
import RiiiverSDK
//import Alamofire

internal class ERAuthFacebookResultImpl : ERAuthFacebookResult {
    var status: ERAuthResultStatus
    
    var token: String?
    
    init(_ status : ERAuthResultStatus) {
        self.status = status
    }
    
    init(error: Error) {
        self.status = .failed(error)
    }
    
    init(token: String) {
        self.token = token
        self.status = .success
    }
}

internal class ERAuthGoogleResultImpl : ERAuthGoogleResult {
    var status: ERAuthResultStatus
    
    var token: String?
    
    init(_ status : ERAuthResultStatus) {
        self.status = status
    }
    
    init(error: Error) {
        self.status = .failed(error)
    }
    
    init(token: String) {
        self.token = token
        self.status = .success
    }
}

internal class ERAuthVendorResultImpl : ERAuthVendorResult {
    var status: ERAuthResultStatus
    
    var params : [String: String]?
    
    init(error: Error) {
        self.status = .failed(error)
    }
    
    init(_ params: [String: String]) {
        self.status = .success
        self.params = params
    }
}

internal class ERAuthDelegateImpl : ERAuthDelegate {
    /// Facebook SDKのLoginManager
    internal var facebookLogin : LoginManager
    /// Facebook AppID
    internal var facebookAppId : String
    /// Facebookのpermissions
    internal var permissions : [String]
    
    /// GIDSignIn オブジェクト
    internal var gidSignIn: GIDSignIn
    /// GIDSignIn用デリゲート
    private  var gidDelegate : ERSDKLoginGoogleDelegate
    
    init() {
        self.facebookLogin = LoginManager()
        
        let facebookDictString = ERAuth.sharedInstance.getConfig("FacebookSignIn")
        let facebookDict : JSON = JSON.init(parseJSON: facebookDictString!)
        self.facebookAppId = facebookDict["AppId"].stringValue
        
        self.permissions = (facebookDict["Permissions"].stringValue).components(separatedBy: ",")
        
        self.gidSignIn = GIDSignIn.sharedInstance()
        
        let googleDictString = ERAuth.sharedInstance.getConfig("GoogleSignIn")
        let googleDict : JSON = JSON.init(parseJSON: googleDictString!)
        let googleClientId : String = googleDict["ClientId"].stringValue
        
        self.gidSignIn.clientID = googleClientId
        self.gidSignIn.scopes = [ "profile", "email", "openid" ]
        
        self.gidDelegate = ERSDKLoginGoogleDelegate()
        
        self.gidSignIn.delegate = self.gidDelegate
    }
    
    func refreshFacebook(completion: @escaping (ERAuthFacebookResult) -> Void) {
        if let _ = AccessToken.current {
            AccessToken.refreshCurrentAccessToken {
                (_, _, err: Error?) -> Void in
                if (err != nil) {
                    completion(ERAuthFacebookResultImpl(error: err!))
                } else {
                    let token = AccessToken.current!.tokenString
                    completion(ERAuthFacebookResultImpl(token: token))
                }
            }
            return
        } else {
            completion(ERAuthFacebookResultImpl(ERAuthResultStatus.notloggedin))
        }
    }
    
    func loginFacebook(viewController: UIViewController?, completion: @escaping (ERAuthFacebookResult) -> Void) {
        self.facebookLogin.logIn(permissions: self.permissions, from: viewController) {
            (result,err) -> Void in
            if err != nil {
                completion(ERAuthFacebookResultImpl(error: err!))
                return
            }
            if result!.isCancelled {
                completion(ERAuthFacebookResultImpl(.cancelled))
                return
            }
            completion(ERAuthFacebookResultImpl(token: result!.token!.tokenString))
        }
    }
    
    func logoutFacebook() {
        self.facebookLogin.logOut()
    }
    
    func refreshGoogle(completion: @escaping (ERAuthGoogleResult) -> Void) {
        if self.gidSignIn.hasPreviousSignIn() {
            self.gidDelegate.googleViewController = nil
            self.gidDelegate.completion = completion
            self.gidSignIn.restorePreviousSignIn()
        } else {
            completion(ERAuthGoogleResultImpl(ERAuthResultStatus.notloggedin))
        }
    }
    
    func loginGoogle(viewController: UIViewController?, completion: @escaping (ERAuthGoogleResult) -> Void) {
        self.gidSignIn.presentingViewController = viewController
        self.gidDelegate.googleViewController = viewController
        self.gidDelegate.completion = completion
        self.gidSignIn.signIn()
    }
    
    func logoutGoogle() {
        GIDSignIn.sharedInstance()?.signOut()
    }
    
    var loadAppleToken : Bool = false
    var appleAccessToken : String?
    var appleRefreshToken : String?
    var appleIdToken : String?
    var appleExpiresAt : Date?

    static func loadApplePrivateKey(keyFile: String) -> Data? {
        let path = Bundle.main.path(forResource: keyFile, ofType: ".p8")
        let url = URL(fileURLWithPath: path!)
        return try? Data(contentsOf: url)
    }
    
    func checkSignInWithApple(with: @escaping (ERAuthAppleParams?) -> Void) {
        if !self.loadAppleToken {
            if let jsonString = ERAuth.sharedInstance.readSignInWithAppleParams() {
                let json = JSON.init(parseJSON: jsonString)
                self.appleAccessToken = json["accessToken"].string
                self.appleRefreshToken = json["refreshToken"].string
                self.appleIdToken = json["idToken"].string
                if let exp = json["expiresAt"].int64 {
                    self.appleExpiresAt = Date(timeIntervalSince1970: TimeInterval(exp))
                }
            }
            self.loadAppleToken = true
        }
        
        let congString = ERAuth.sharedInstance.getConfig("SignInWithApple")
        let conf : JSON = JSON.init(parseJSON: congString!)
        let teamId = conf["teamId"].string!
        let serviceId = conf["serviceId"].string!
        let keyId = conf["keyId"].string!
        let privateKey = ERAuthDelegateImpl.loadApplePrivateKey(keyFile: conf["privateKey"].string!)
        let serviceSecret = try? ERAuth.sharedInstance.appleClientSecret(teamId: teamId, clientId: serviceId, keyId: keyId, privateKey: privateKey)

        let isNativeLogin = false // TODO: ネイティブログインがあるかどうかで適宜変更
        var appleParams : ERAuthAppleParams?
        if serviceSecret == nil {
        } else if #available(iOS 13.0, *), isNativeLogin {
            // iOS 13以降でAppleのネイティブログインを使用している場合
            // refreshに使用するIDとしてアプリのclient_idを設定する
            // 検証用にclient_id用のsecretも必要になるため生成する
            let refreshId = conf["clientId"].string!
            let clientSecret = try? ERAuth.sharedInstance.appleClientSecret(teamId: teamId, clientId: refreshId, keyId: keyId, privateKey: privateKey)
            appleParams = ERAuthAppleParams(serviceId: serviceId, serviceSecret: serviceSecret!, idToken: self.appleIdToken, refreshToken: self.appleRefreshToken, refreshId: refreshId, clientSecret: clientSecret)
        } else {
            // iOS 12以前か、AppleのWebログインを使用している場合
            // refreshに使用するIDは未指定でよい。Riiiver Developrsにて設定したserviceIdが使用されます
            // client_idを使用していないため、secretも必要ありません
            appleParams = ERAuthAppleParams(serviceId: serviceId, serviceSecret: serviceSecret!, idToken: self.appleIdToken, refreshToken: self.appleRefreshToken, refreshId: nil, clientSecret: nil)

        }
        with(appleParams)
    }
    
    func clearAppleTokens() {
        do {
            self.appleAccessToken = nil
            self.appleRefreshToken = nil
            self.appleIdToken = nil
            self.appleExpiresAt = nil
            let params : String = ""
            try ERAuth.sharedInstance.writeSignInWithAppleParams(params: params)
        }
        catch let err {
            // TODO
        }
    }
    
    func updateAppleTokens(accessToken: String?, refreshToken: String?, idToken: String?, expiresAt: Date?) {
        do {
            self.appleAccessToken = accessToken
            self.appleExpiresAt = expiresAt
            if idToken != nil {
                self.appleIdToken = idToken
            }
            if refreshToken != nil {
                self.appleRefreshToken = refreshToken
            }
            var params : JSON = JSON()
            if let token = self.appleAccessToken {
                params["accessToken"] = JSON(token)
            }
            if let token = self.appleRefreshToken {
                params["refreshToken"] = JSON(token)
            }
            if let token = self.appleIdToken {
                params["idToken"] = JSON(token)
            }
            if let exp = self.appleExpiresAt {
                let expAt = Int64(exp.timeIntervalSince1970)
                params["expiresAt"] = JSON(expAt)
            }
            try ERAuth.sharedInstance.writeSignInWithAppleParams(params: params.rawString())
        }
        catch let err {
            // TODO
        }
    }
    
    func loginVendor(completion: @escaping (ERAuthVendorResult) -> Void) {
        let result = [ "region" : "JP" ]
        completion(ERAuthVendorResultImpl(result))
    }
    
    func interceptApplication(_ application: UIApplication, open url: URL,
                              sourceApplication: String?, annotation: Any) -> Bool {
        if (GIDSignIn.sharedInstance().handle(url)) {
            return true
        }
        if (ApplicationDelegate.shared.application(application, open: url, options: [UIApplication.OpenURLOptionsKey.sourceApplication : sourceApplication as Any ])) {
            return true
        }
        return false
    }
    
    func interceptApplication(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?, resumeSessionWithCompletionHandler:((Any?,Error?)->Void)?) -> Bool {
        return ApplicationDelegate.shared.application(application, didFinishLaunchingWithOptions: launchOptions)
    }
}

/// Googleログイン用デリゲート
private class ERSDKLoginGoogleDelegate : NSObject, GIDSignInDelegate {
    
    /// ビューコントローラー
    internal weak var googleViewController : UIViewController?
    
    /// 完了/エラー処理コールバック関数 引数1: エラーオブジェクト (nil:正常)
    internal var completion : ((ERAuthGoogleResult) -> Void)?
    
    /// SignIn完了コールバック
    /// SignInの完了で、Google SignIn SDK が呼び出します
    ///
    /// - Parameters:
    ///   - signIn: GIDSignInオブジェクト
    ///   - user: GIDGoogleUserオブジェクト
    ///   - error: エラーオブジェクト
    public func sign(_ signIn: GIDSignIn!, didSignInFor user: GIDGoogleUser!, withError error: Error!) {
        if (error != nil) {
            self.completion?(ERAuthGoogleResultImpl(error: error))
            return
        }
        if user.authentication.clientID != signIn.clientID {
            signIn.signOut()
            self.completion?(ERAuthGoogleResultImpl(.notloggedin))
            return
        }
        self.completion?(ERAuthGoogleResultImpl(token: user.authentication.idToken))
    }
    
    /// SignOut完了コールバック
    /// SignOutはここでやってないので、特に何もしない
    ///
    /// - Parameters:
    ///   - signIn: GIDSignInオブジェクト
    ///   - user: GIDGoogleUserオブジェクト
    ///   - error: エラーオブジェクト
    public func sign(_ signIn: GIDSignIn!, didDisconnectWith user: GIDGoogleUser!, withError error: Error!) {
    }
    
    /// SignIn Hookコールバック
    /// Hookする必要がないので、とくに何もしない
    ///
    /// - Parameters:
    ///   - signIn: GIDSignInオブジェクト
    ///   - error: エラーオブジェクト
    public func sign(inWillDispatch signIn: GIDSignIn!, error: Error!) {
    }
    
    /// SignIn UI 表示用コールバック
    /// viewControllerの表示時に呼び出される。
    /// 特殊な処理はせず、presentしとく。
    ///
    /// - Parameters:
    ///   - signIn: GIDSignInオブジェクト
    ///   - viewController: 表示されるビューコントローラー
    public func sign(_ signIn: GIDSignIn!, present viewController: UIViewController!) {
        self.googleViewController?.present(viewController, animated: false, completion: nil)
    }
    
    /// SignIn UI 消去用コールバック
    /// viewControllerを消去時に呼び出される。
    /// 特殊な処理はせずに、dismissしとく。
    ///
    /// - Parameters:
    ///   - signIn: GIDSignInオブジェクト
    ///   - viewController: 消去されるビューコントローラー
    public func sign(_ signIn: GIDSignIn!, dismiss viewController: UIViewController!) {
        self.googleViewController?.dismiss(animated: false, completion: nil)
    }
    
}

