//
//  ERSDKLogin.swift
//  sample
//
//  Created by CITIZEN Watch Co.,Ltd on 2019/10/28.
//  Copyright © 2018年 veldt. All rights reserved.
//

import Foundation
import WebKit
import RiiiverSDK

public typealias ERSDKUserProfile = ERUserProfile

public typealias ERSDKUserManager = ERUserManager

public typealias ERSDKLoginAWSToken = ERAuthToken

/// 認証画面用デリゲート
public protocol ERSDKLoginWebViewDelegate {
    
    /// 認証処理終了
    ///
    /// - Parameter viewController: ビューコントローラー
    func onCloseViewController(_ viewController: UIViewController)
    
    /// エラー処理
    ///
    /// - Parameter err: エラー
    func onError(_ err: Error)
}

/// ストア画面用デリゲート
public protocol ERSDKStoreWebViewDelegate {
    
    /// ストアのクローズ
    func onClose()
    
    /// エラー処理
    ///
    /// - Parameter err: エラー
    func onError(_ err: Error)
}

/// 認証API
public class ERSDKLogin {
    /// Singletonオブジェクト
    public static let sharedInstance = ERSDKLogin()
    
    private init() {
        ERAuth.sharedInstance.setupDelegate(ERAuthDelegateImpl())
    }
    
    /// ログイン状態のチェック
    ///
    /// - Returns: true/ログイン中, false/未ログイン
    public func isLoggedIn() -> Bool {
        return ERAuth.sharedInstance.isLoggedIn()
    }
    
    /// ecoSystemIdの取得
    public func getEcoSystemId() -> String? {
        return ERAuth.sharedInstance.getEcoSystemId()
    }
    
    private class ERSDKLoginWebViewDelegate_ : ERAuthWebViewDelegate {
        private var delegate: ERSDKLoginWebViewDelegate?
        
        init(_ delegate: ERSDKLoginWebViewDelegate?) {
            self.delegate = delegate
        }
        
        func onCloseViewController(_ viewController: UIViewController?) {
            self.delegate?.onCloseViewController(viewController!)
        }
        
        func onError(_ err: Error) {
            self.delegate?.onError(err)
        }
    }
    
    /// 認証画面設定
    ///
    /// - Parameters:
    ///   - viewController: ビューコントローラー
    ///   - webView: WebView
    ///   - vendorId: ベンダーID
    ///   - deviceIds: デバイスIDの配列
    ///   - appName: アプリ名
    ///   - delegate: 認証画面デリゲート
    public func setWebView(viewController:UIViewController, webView: WKWebView, vendorId: String, deviceIds: [String]?, appName: String, delegate: ERSDKLoginWebViewDelegate?) {
        let delegate_ = ERSDKLoginWebViewDelegate_(delegate)
        ERAuth.sharedInstance.openLogin(viewController: viewController, webView: webView, vendorToken: nil, deviceIds: deviceIds, delegate: delegate_)
    }
    
    private class ERSDKStoreWebViewDelegate_ : ERAuthWebViewDelegate {
        private var delegate: ERSDKStoreWebViewDelegate
        
        init(_ delegate: ERSDKStoreWebViewDelegate) {
            self.delegate = delegate
        }
        
        func onCloseViewController(_ viewController: UIViewController?) {
            self.delegate.onClose()
        }
        
        func onError(_ err: Error) {
            self.delegate.onError(err)
        }
    }
    
    /// ビューコントローラー取得
    /// 表示中のビューコントローラーを取得します。
    ///
    /// - Parameter view: ビュー
    /// - Returns: ビューコントローラー
    private func getViewController(_ view: UIView) -> UIViewController {
        var responder : UIResponder? = view
        while !(responder is UIViewController) {
            responder = responder?.next
            if nil == responder {
                break
            }
        }
        return (responder as! UIViewController)
    }
    

    /// ストア画面設定
    ///
    /// - Parameters:
    ///   - webView: WebView
    ///   - vendorId: ベンダーID
    ///   - vendorToken: ベンダートークン
    ///   - deviceIds: デバイスIDの配列
    ///   - delegate: ストア画面デリゲート
    public func openStore(webView: WKWebView, vendorId: String, vendorToken: String?, deviceIds: [String]?, appName: String, delegate: ERSDKStoreWebViewDelegate) {
        let delegate_ = ERSDKStoreWebViewDelegate_(delegate)
        ERAuth.sharedInstance.openStore(viewController: self.getViewController(webView), webView: webView, vendorToken: vendorToken, deviceIds: deviceIds, delegate: delegate_)
    }
    
    /// 認証トークン取得
    ///
    /// - Parameter handler: 完了コールバック (エラー, 認証トークン)
    public func getAWSToken(handler: ((Error?,ERSDKLoginAWSToken?) -> Void)?) {
        ERAuth.sharedInstance.getToken {
            (err: Error?, token: ERAuthToken?) -> Void in
            if (err != nil) {
                handler?(err, nil)
                return
            }
            handler?(nil, token!)
        }
    }
    
    /// ログアウト処理
    ///
    /// - Parameter completion: コールバック 引数1: エラーオブジェクト (nilのとき正常)
    public func logout(completion:((Error?)->Void)?) {
        return ERAuth.sharedInstance.logout(completion: completion)
    }
    
    /// パスワードを変更する
    ///
    /// - Parameters:
    ///   - newPassword: 新パスワード
    ///   - completion: コールバック 引数1: エラーオブジェクト (nilのとき正常)
    public func changePassword(oldPassword: String, newPassword: String, completion: ((Error?)->Void)?) {
        return ERAuth.sharedInstance.changePassword(oldPassword:oldPassword, newPassword: newPassword, completion: completion)
    }
    
    /// メールアドレスを変更する
    ///
    /// - Parameters:
    ///   - newEmail: メールアドレス
    ///   - completion: コールバック 引数1: エラーオブジェクト (nilのとき正常)
    public func changeEmail(newEmail: String, completion: ((Error?)->Void)?) {
        return ERAuth.sharedInstance.changeEmail(newEmail:newEmail, completion: completion)
    }
    
    /// appdelegate.application:url:sourceApplication:annotation:メソッド用フック
    ///
    /// - Parameters:
    ///   - application: application
    ///   - url: url
    ///   - sourceApplication: sourceApplication
    ///   - annotation: annotation
    /// - Returns: true: フックする false: フックしない
    public func interceptApplication(_ application: UIApplication, open url: URL,
                                     sourceApplication: String?, annotation: Any) -> Bool {
        return ERAuth.sharedInstance.interceptApplication(application, open: url, sourceApplication: sourceApplication, annotation: annotation)
    }
    
    /// appdelegate.application:launchOptions:メソッド用フック
    ///
    /// - Parameters:
    ///   - application: application
    ///   - launchOptions: launchOptions
    /// - Returns: true: フックする false: フックしない
    public func interceptApplication(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        return ERAuth.sharedInstance.interceptApplication(application, didFinishLaunchingWithOptions: launchOptions)
    }
    
    
    /// appdelegate.application:launchOptions:resumeSessionWithCompletionHandler:メソッド用フック
    ///
    /// - Parameters:
    ///   - application: application
    ///   - launchOptions: launchOption
    ///   - resumeSessionWithCompletionHandler: ハンドラ
    /// - Returns: true: フックする false: フックしない
    public func interceptApplication(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?, resumeSessionWithCompletionHandler:((Any?,Error?)->Void)?) -> Bool {
        return ERAuth.sharedInstance.interceptApplication(application, didFinishLaunchingWithOptions: launchOptions, resumeSessionWithCompletionHandler: resumeSessionWithCompletionHandler)
    }
}
