//
//  RiiiverSDKManager.swift
//  sample
//
//  Created by CITIZEN Watch Co.,Ltd on 2019/10/28.
//  Copyright © 2019 CITIZEN Watch Co.,Ltd. All rights reserved.
//

import Foundation
import WebKit

import RiiiverSDK

/// The class for managing Riiiver SDK information.  -  RiiiverSDKから取得した情報を管理
class RiiiverSDKManager : ERSDKDelegate {
    /// Singleton
    static let shared = RiiiverSDKManager()
    
    /// accecss Token  -  認証トークン
    public private(set) var accessToken: String?
    
    /// ID Token
    public private(set) var idToken: String?
    
    private init() {
        ERSDK.sharedInstance.delegate = self
    }
    
    /// 認証済みかどうか (true:認証済み、false:未認証)
    /// Is this user authenticated?  true:  Yes,  false: No
    public var authenticated: Bool {
        return ERSDKLogin.sharedInstance.isLoggedIn()
    }
    
    /// RiiiverSDKからWeb認証画面表示する
    ///  set a web view to be  invoked by Riiiver SDK for the web page authentication.
    /// - Parameters:
    ///   - completion:
    func setWebView(viewController: UIViewController, webView: WKWebView, delegate: ERSDKLoginWebViewDelegate) {
        let vendorId:String = AppManager.shared.getVendorId()
        let deviceId:[String] = AppManager.shared.getDeviceId()
        let appName = AppManager.shared.getAppName()
        // Display the authentication web page.  -  Web認証画面表示
        ERSDKLogin.sharedInstance.setWebView(viewController: viewController, webView: webView, vendorId: vendorId, deviceIds: deviceId, appName: appName, delegate: delegate)
    }
    
    /// RiiiverSDKから認証トークンを取得して保持する
    ///  Get the auth token and cache it.
    /// - Parameters:
    ///   - completion:
    ///     - isSuccess: true > Success to get the authentication info.  false > Failed to get the auth info.  認証情報の取得が成功した場合はtrue、失敗した場合はfalse
    ///     - error:  the error information from Riiiver SDK.  -   RiiiverSDKから返却されたerror情報
    func getAWSToken(completion: @escaping (( _ isSuccess: Bool, _ error: Error?) -> Void)) {
        // Check for the state of the authentication inside of RiiiverSDK.  -  RiiiverSDKの認証状態をチェック
        if (authenticated == false ){
            //  When failed to authenticate, quit this because we couldn't get auth token.  -  未認証の場合、認証トークンを取得できないので終了
            print("getAWSToken failed. Unauthenticated.")
            completion(false, nil)
            return
        }
        
        // RiiiverSDKから認証トークンを取得 - Get the token from Riiiver SDK.
        ERSDKLogin.sharedInstance.getAWSToken(handler: { (error, token) in
            
            guard error == nil else {
                print("getAWSToken failed. reason:\(error?.localizedDescription ?? "")")
                completion(false, error)
                return
            }
            
            guard let token = token else {
                print("getAWSToken failed. reason: No token")
                completion(false, error)
                return
            }
            
            guard let accessToken = token.accessToken else {
                print("getAWSToken failed. reason: No access-token")
                completion(false, error)
                return
            }
            // 取得した情報を保持 - store the token into app side.   (But never use them, in this app.)
            self.accessToken = token.accessToken
            self.idToken = token.idToken
            
            print("getAWSToken succeeded. accessToken: \(accessToken)")
            completion(true, error)
        })
    }
    
    /// RiiiverSDKからiiideaStore画面表示する
    ///  Display Riiiver Store page via Riiiver SDK.
    /// - Parameters:
    ///   - webView
    ///   - delegate
    func openStore(webView: WKWebView, delegate: ERSDKStoreWebViewDelegate) {
        let vendorId:String = AppManager.shared.getVendorId()
        let deviceId:[String] = AppManager.shared.getDeviceId()
        let appName = AppManager.shared.getAppName()
        ERSDKLogin.sharedInstance.openStore(webView: webView, vendorId: vendorId, vendorToken: nil, deviceIds: deviceId, appName: appName, delegate: delegate)
    }
    
    
    /// RiiiverSDKから My iiidea 一覧を取得する
    /// Get the my iiidea list from RiiiverSDK.
    /// - Parameters:
    ///   - completion:
    ///     - isSuccess: My iiidea 一覧を取得が成功した場合はtrue、失敗した場合はfalse
    ///     - error: RiiiverSDKから返却されたerror情報
    func getMyApplets(completion: @escaping ((_ isSuccess: Bool, _ iiideas: [ERApplet]? , _ error: Error?) -> Void)) {
        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        self.getAWSToken { (isSuccess, error) in
            if (isSuccess == false) {
                print("getMyApplets failed. reason: No token")
                completion(false, nil, error)
                return
            }
            // RiiiverSDKから My iiidea 一覧を取得 - get my iiideas from Riiiver SDK.
            ERSDK.sharedInstance.getMyApplets(deviceIds: AppManager.shared.getDeviceId()) { (myApplets, error) in
                
                // Note: エラーがあっても取得できた My iiidea は返却されるため処理を継続することは可能 - Here, we can get the iiidea even if we get some errors.
                if let error = error {
                    print("getMyApplets failed. reason: error:\(error)")
                }

                print("getMyApplets succeeded. My iiidea(\(myApplets?.count ?? 0))")
                completion(true, myApplets, error)
            }
        }
    }
    
    /// 指定したiiideaをアクティベートする
    /// Activate an terget iiidea
    /// - Parameters:
    ///   - iiidea: アクティベートするiiidea
    ///   - completion:
    ///     - isSuccess:アクティベートが成功した場合はtrue、失敗した場合はfalse
    ///     - error: RiiiverSDKから返却されたerror情報
    func activate(iiidea: ERApplet, completion: @escaping ((_ isSuccess: Bool, _ error: Error?) -> Void)) {
        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        self.getAWSToken { (isSuccess, error) in
            if (isSuccess == false) {
                print("activate failed. reason: No token")
                completion(false, error)
                return
            }
            // Activate the iiidea via Riiiver SDK.
            iiidea.activate() { unauthPermissions, serviceProxyData, error in
                guard unauthPermissions == nil || unauthPermissions == [] else {
                    print("activate failed. reason:No permission")
                    completion(false, error)
                    return
                }
                guard error == nil else {
                    let riiiverError:ERSDKError = error as! ERSDKError
                    print("activate failed. reason: \(riiiverError.rawValue)")
                    completion(false, error)
                    return
                }
                print("activate succeeded.")
                completion(true, error)
            }
        }
    }
    
    /// 指定したiiideaをディアクティベートする
    ///  Deactivate the target iiidea.
    /// - Parameters:
    ///   - iiidea: ディアクティベートするiiidea  - target iiidea to deactivate.
    ///   - completion:
    ///     - isSuccess:ディアクティベートが成功した場合はtrue、失敗した場合はfalse
    ///     - error: RiiiverSDKから返却されたerror情報
    func deactivate(iiidea: ERApplet, completion: @escaping ((_ isSuccess: Bool, _ error: Error?) -> Void)) {
        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        self.getAWSToken { (isSuccess, error) in
            if (isSuccess == false) {
                print("deactivate failed. reason: No token")
                completion(false, error)
                return
            }
            // Deactivate the iiidea via Riiiver SDK.
            iiidea.deactivate()
            print("deactivate succeeded.")
            completion(true, error)
        }
    }
    
    /// 指定したiiideaを削除する
    /// Remove the target iiidea.
    /// - Parameters:
    ///   - iiidea: 削除するiiidea  - target iiidea to remove.
    ///   - completion:
    ///     - isSuccess:削除が成功した場合はtrue、失敗した場合はfalse
    ///     - error: RiiiverSDKから返却されたerror情報
    func removeMyIiidea(iiidea: ERApplet, completion: @escaping ((_ isSuccess: Bool, _ error: Error?) -> Void)) {
        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        self.getAWSToken { (isSuccess, error) in
            if (isSuccess == false) {
                print("removeMyIiidea failed. reason: No token")
                completion(false, error)
                return
            }
            
            ERSDK.sharedInstance.removeMyApplet(applet: iiidea) {(error) in
                guard error == nil else {
                    let riiiverError:ERSDKError = error as! ERSDKError
                    print("removeMyIiidea failed. reason: \(riiiverError.rawValue)")
                    completion(false, error)
                    return
                }
                print("removeMyIiidea succeeded.")
                completion(true, error)
            }
        }
    }
    
    /// RiiiverSDKで生成したUUIDを取得する
    func getUuid() -> UUID? {
        let uuid = ERSDK.sharedInstance.getUuid()
        print("getUuid : \(uuid)" )
        return uuid
    }
    
    /// iiideaが実行開始した時に通知する
    /// - Parameters:
    ///   - applet: 起動したiiideaのインスタンス
    ///   - error: RiiiverSDKから返却されたerror情報
    func startApplet(applet: ERApplet, sdkError: ERSDKError) {
        print("startApplet : appletId=\(applet.getId()), appletTitle=\(applet.applet.getTitle()), sdkError=\(sdkError)" )
    }
    
    /// iiideaが実行終了した時に通知する
    /// - Parameters:
    ///   - applet: 終了したiiideaのインスタンス
    ///   - error: iiideaが正しく終了しなかった時のエラー
    ///   - reachBlock: iiideatが終了した時の最終到達Block
    ///   - output: iiideaが正しく終了した際の、最後に到達したPieceが出力したデータ
    func endApplet(applet: ERApplet, sdkError: ERSDKError, reachBlock: ReachBlock, output: Dictionary<String, Any>?) {
        print("endApplet : appletId=\(applet.getId()), appletTitle=\(applet.applet.getTitle()), sdkError=\(sdkError), reachBlock=\(reachBlock), output=\(String(describing: output))")
    }
}
